# -*- coding: utf-8 -*-
from setuphelpers import *
import waptguihelper
import json


def get_persistent_package_file(fname):
    if isdir(makepath(os.getcwd(), "WAPT", "persistent")):
        return makepath(os.getcwd(), "WAPT", "persistent", fname)
    else:
        return makepath(WAPT.persistent_root_dir, control.package_uuid, fname)


config_file = "mesh-conf.json"
mesh_config_dict_old = json_load_file(get_persistent_package_file(config_file))


def update_package():
    # Declaring local variables
    package_updated = False

    # Gathering informations for the package
    if "template" in control.package:
        package = waptguihelper.input_dialog(
            control.package, "Please change the template suffix with a term corresponding to your Mesh Agent", control.package
        )
        control.package = package
        control.save_control_to_wapt()

    dialog_response = waptguihelper.grid_dialog(
        "Adapt your Agent parameters (corresponding to: meshcentral-data/config.json)",
        format_dict_to_grid(mesh_config_dict_old),
        0,
        '{"columns":[{"propertyname":"parameter","datatype":"String","required":false,"readonly":true,"width":153},{"propertyname":"value","datatype":"String","required":false,"readonly":false,"width":172}]}',
    )
    json_write_file(get_persistent_package_file(config_file), format_grid_to_dict(dialog_response))
    mesh_config_dict = json_load_file(get_persistent_package_file(config_file))

    # Getting binaries
    if glob.glob("*.exe"):
        ask_for_binaries = waptguihelper.message_dialog(control.package, "Do you want to deploy new Mesh Agent binaries ?", waptguihelper.MB_YESNO)
    else:
        ask_for_binaries = 6

    # Deleting binaries
    if ask_for_binaries == 6:
        for f in glob.glob("*.exe") + glob.glob("*.msi"):
            print("Removing: %s" % f)
            remove_file(f)

        bin_path_x86 = waptguihelper.filename_dialog(
            "Please provide x86 Mesh Agent",
            "",
            "%s32-%s.exe" % (mesh_config_dict["mesh_fileName"].replace(" ", ""), mesh_config_dict["mesh_deviceGroup"].replace(" ", "")),
            "EXE Files|*.exe",
        )
        bin_path_x64 = waptguihelper.filename_dialog(
            "Please provide x64 Mesh Agent",
            "",
            "%s64-%s.exe" % (mesh_config_dict["mesh_fileName"].replace(" ", ""), mesh_config_dict["mesh_deviceGroup"].replace(" ", "")),
            "EXE Files|*.exe",
        )
        bin_name_x64 = bin_path_x64.split(os.sep)[-1].split(".")[0]
        if bin_path_x86:
            bin_name_x86 = bin_path_x86.split(os.sep)[-1].split(".")[0]
            filecopyto(bin_path_x86, basedir)
            if not bin_name_x86 in control.impacted_process:
                control.impacted_process = control.impacted_process + "," + bin_name_x86
                control.save_control_to_wapt()
        filecopyto(bin_path_x64, basedir)
        if not bin_name_x64 in control.impacted_process:
            control.impacted_process = control.impacted_process + "," + bin_name_x64
            control.save_control_to_wapt()

        # Changing version of the package
        version = get_file_properties(bin_path_x64).get("FileVersion", control.get_software_version())
        if Version(version) > Version(control.get_software_version()):
            print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
            package_updated = True
        else:
            print("Software version up-to-date (%s)" % Version(version))
        control.set_software_version(version)
        control.save_control_to_wapt()
    if not mesh_config_dict["mesh_fileName"] in control.impacted_process:
        control.impacted_process = control.impacted_process + "," + mesh_config_dict["mesh_fileName"]
        control.save_control_to_wapt()

    # Editing control
    if not "template" in control.package:
        control.name = mesh_config_dict["mesh_displayName"]
        control.description = mesh_config_dict["mesh_displayName"]
        control.save_control_to_wapt()

    # Validating or not update-package-sources
    return package_updated


def format_dict_to_grid(dict_data, key_name="parameter", value_name="value"):
    dict_list_data = []
    for section in dict_data:
        dict_list_data.append({key_name: section, value_name: dict_data[section]})
    return json.dumps(dict_list_data)


def format_grid_to_dict(dict_list_data, key_name="parameter", value_name="value"):
    dict_data = {}
    for entry in dict_list_data:
        dict_data.update({entry[key_name]: entry[value_name]})
    return dict_data
